#include "bpmcalculatorwidget.h"
#include "tools/bpmcalculator.h"

BPMCalculatorWidget::BPMCalculatorWidget(const std::shared_ptr<BPMCalculator>& bpm_calculator, const std::shared_ptr<sf::Font>& font) :
    Window("BPM Calculation", font),
    _bpm_calculator(bpm_calculator),
    _slider(std::make_shared<BPMSlider>())
{
    _bpm_value.setFont(*_font);
    _bpm_value.setCharacterSize(40);
    _bpm_value.setFillColor(sf::Color::Black);
    _bpm_value.setString("--");
}

void BPMCalculatorWidget::input(const sf::Event& event)
{
    switch (event.type)
    {
    default:
        break;

    case sf::Event::KeyPressed:
        if (event.type == sf::Event::KeyPressed && event.key.code == sf::Keyboard::Space)
        {
            _bpm_calculator->click();
        }
        break;
    }

    Window::input(event);
}

void BPMCalculatorWidget::update(const sf::Time& dt)
{
    Window::update(dt);

    const auto approximation = _bpm_calculator->fetchCurrentBPMApproximation();
    if (approximation != 0)
    {
        _bpm_value.setString(std::to_string(approximation));

        const microsec until_beat = _bpm_calculator->fetchTimeUntilNextBeat();
        const microsec beat_interval = _bpm_calculator->fetchBeatInterval();

        const auto time_relation = beat_interval / until_beat;
        const auto slider_path_left = Window::rect().width / time_relation;

        _slider->setTickPosition(slider_path_left);
    }
}

void BPMCalculatorWidget::draw(sf::RenderTarget& target, sf::RenderStates states) const
{
    Window::draw(target, states);

    if (_is_visible)
    {
        _slider->draw(target, states);
        _button_start->draw(target, states);
        target.draw(_bpm_value, states);
    }
}

void BPMCalculatorWidget::setRect(const sf::FloatRect& rect)
{
    Window::setRect(rect);
    _slider->setRect(sf::FloatRect{0, 0, rect.width / 8 * 6, 100});
    _slider->setPosition({_window_content.getGlobalBounds().left + rect.width / 8,
                          _window_content.getGlobalBounds().top + rect.height / 8 * 3});

    _button_start->setRect(sf::FloatRect{0, 0, rect.width / 10 * 3, 30});
    _button_start->setPosition({_window_content.getGlobalBounds().left + rect.width / 7,
                                _window_content.getGlobalBounds().top + _window_content.getGlobalBounds().height - 40});

    _bpm_value.setPosition({_window_content.getGlobalBounds().left + rect.width / 8,
                            _window_content.getGlobalBounds().top + rect.height / 8 });
}

void BPMCalculatorWidget::move(const sf::Vector2f &delta)
{
    Window::move(delta);
    _slider->move(delta);
    _bpm_value.move(delta);
}

void BPMCalculatorWidget::setPosition(const sf::Vector2f &position)
{
    Window::setPosition(position);
}

void BPMCalculatorWidget::init()
{
    auto& bpm_calculator = _bpm_calculator;

    _button_start = std::make_shared<PushButton>("Start", _font);
    _button_start->setCallback([bpm_calculator, button_start=_button_start]()
    {
        bpm_calculator->music()->play();
        bpm_calculator->start();
        button_start->setVisibility(false);
    });

    addChild(_button_start);
}
